@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2005-2007 Sun Microsystems Inc.
+# Copyright 2005-2008 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -14,16 +14,16 @@
 #
 # You should have received a copy of the GNU Library General Public
 # License along with this library; if not, write to the
-# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-# Boston, MA 02111-1307, USA.
+# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
+# Boston MA  02110-1301 USA.
 
 """Provides support for defining keybindings and matching them to input
 events."""
 
-__id__        = "$Id: keybindings.py 3272 2007-11-29 18:01:28Z richb $"
-__version__   = "$Revision: 3272 $"
-__date__      = "$Date: 2007-11-29 13:01:28 -0500 (Thu, 29 Nov 2007) $"
-__copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
+__id__        = "$Id$"
+__version__   = "$Revision$"
+__date__      = "$Date$"
+__copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 try:
@@ -39,6 +39,7 @@ except ImportError:
 import pyatspi
 import debug
 import settings
+import orca_state
 
 from orca_i18n import _           # for gettext support
 
@@ -50,7 +51,7 @@ def getAllKeysyms(keysym):
     that is mapped to the given keysym.  This allows us, for example,
     to determine that the key bound to KP_Insert is also bound to KP_0."""
 
-    if not _keysymsCache.has_key(keysym):
+    if keysym not in _keysymsCache:
         # The keysym itself is always part of the list.
         #
         _keysymsCache[keysym] = [keysym]
@@ -118,7 +119,7 @@ def getKeycode(keysym):
     if not keysym:
         return 0
 
-    if not _keycodeCache.has_key(keysym):
+    if keysym not in _keycodeCache:
         keymap = gtk.gdk.keymap_get_default()
 
         # Find the numerical value of the keysym
@@ -152,7 +153,7 @@ def getModifierNames(mods):
     """
 
     text = ""
-    if mods & (1 << settings.MODIFIER_ORCA):
+    if mods & settings.ORCA_MODIFIER_MASK:
         text += _("Orca") + "+"
     #if mods & (1 << pyatspi.MODIFIER_NUMLOCK):
     #    text += _("Num_Lock") + "+"
@@ -173,12 +174,12 @@ def getModifierNames(mods):
         text += _("Meta2") + "+"
     #if mods & (1 << pyatspi.MODIFIER_META):
     #    text += _("Meta") + "+"
-    if mods & (1 << pyatspi.MODIFIER_ALT):
+    if mods & settings.ALT_MODIFIER_MASK:
         # Translators: this is presented in a GUI to represent the
         # "left alt" modifier.
         #
         text += _("Alt_L") + "+"
-    if mods & (1 << pyatspi.MODIFIER_CONTROL):
+    if mods & settings.CTRL_MODIFIER_MASK:
         # Translators: this is presented in a GUI to represent the
         # "control" modifier.
         #
@@ -188,7 +189,7 @@ def getModifierNames(mods):
         # "caps lock" modifier.
         #
         text += _("Caps_Lock") + "+"
-    if mods & (1 << pyatspi.MODIFIER_SHIFT):
+    if mods & settings.SHIFT_MODIFIER_MASK:
         # Translators: this is presented in a GUI to represent the
         # "shift " modifier.
         #
@@ -200,7 +201,8 @@ class KeyBinding:
     and the InputEventHandler.
     """
 
-    def __init__(self, keysymstring, modifier_mask, modifiers, handler):
+    def __init__(self, keysymstring, modifier_mask, modifiers, handler,
+                 click_count = 1):
         """Creates a new key binding.
 
         Arguments:
@@ -219,6 +221,7 @@ class KeyBinding:
         self.modifier_mask = modifier_mask
         self.modifiers = modifiers
         self.handler = handler
+        self.click_count = click_count
         self.keycode = None
 
     def matches(self, keycode, modifiers):
@@ -249,10 +252,12 @@ class KeyBindings:
     def __str__(self):
         result = "[\n"
         for keyBinding in self.keyBindings:
-            result += "  [%x %x %s %s]\n" % (keyBinding.modifier_mask,
-                                             keyBinding.modifiers,
-                                             keyBinding.keysymstring,
-                                             keyBinding.handler.description)
+            result += "  [%x %x %s %d %s]\n" % \
+                      (keyBinding.modifier_mask,
+                       keyBinding.modifiers,
+                       keyBinding.keysymstring,
+                       keyBinding.click_count,
+                       keyBinding.handler.description)
         result += "]"
         return result
     
@@ -283,10 +288,12 @@ class KeyBindings:
         """Return True if keyBinding is already in self.keyBindings.
 
            The typeOfSearch can be:
-              "strict":      matches description, modifiers, key
+              "strict":      matches description, modifiers, key, and
+                             click count
               "description": matches only description.
-              "keys":        matches the modifiers, key, and modifier mask
-              "keysNoMask":  matches the modifiers and key only
+              "keys":        matches the modifiers, key, and modifier mask,
+                             and click count
+              "keysNoMask":  matches the modifiers, key, and click count
         """
 
         hasIt = False
@@ -300,7 +307,9 @@ class KeyBindings:
                     and (keyBinding.modifier_mask \
                          == newKeyBinding.modifier_mask) \
                     and (keyBinding.modifiers \
-                         == newKeyBinding.modifiers):
+                         == newKeyBinding.modifiers) \
+                    and (keyBinding.click_count \
+                         == newKeyBinding.click_count):
                     hasIt = True
             elif typeOfSearch == "description":
                 if keyBinding.handler.description \
@@ -312,13 +321,17 @@ class KeyBindings:
                     and (keyBinding.modifier_mask \
                          == newKeyBinding.modifier_mask) \
                     and (keyBinding.modifiers \
-                         == newKeyBinding.modifiers):
+                         == newKeyBinding.modifiers) \
+                    and (keyBinding.click_count \
+                         == newKeyBinding.click_count):
                     hasIt = True
             elif typeOfSearch == "keysNoMask":
                 if (keyBinding.keysymstring \
                     == newKeyBinding.keysymstring) \
                     and (keyBinding.modifiers \
-                         == newKeyBinding.modifiers):
+                         == newKeyBinding.modifiers) \
+                    and (keyBinding.click_count \
+                         == newKeyBinding.click_count):
                     hasIt = True
 
         return hasIt
@@ -327,15 +340,35 @@ class KeyBindings:
         """Returns the input handler of the key binding that matches the
         given keycode and modifiers, or None if no match exists.
         """
-        handler = None
+
+        if not orca_state.activeScript:
+            return None
+
+        candidates = []
+        clickCount = orca_state.activeScript.getClickCount()
         for keyBinding in self.keyBindings:
             if keyBinding.matches(keyboardEvent.hw_code, \
                                   keyboardEvent.modifiers):
-                handler = keyBinding.handler
-                if keyBinding.modifier_mask == keyboardEvent.modifiers:
-                    break
+                if keyBinding.modifier_mask == keyboardEvent.modifiers and \
+                   keyBinding.click_count == clickCount:
+                    return keyBinding.handler
+                # If there's no keysymstring, it's unbound and cannot be
+                # a match.
+                #
+                if keyBinding.keysymstring:
+                    candidates.append(keyBinding)
 
-        return handler
+        # If we're still here, we don't have an exact match. Prefer
+        # the one whose click count is closest to, but does not exceed,
+        # the actual click count.
+        #
+        candidates.sort(cmp=lambda x, y: x.click_count - y.click_count,
+                        reverse=True)
+        for candidate in candidates:
+            if candidate.click_count <= clickCount:
+                return candidate.handler
+
+        return None
 
     def consumeKeyboardEvent(self, script, keyboardEvent):
         """Attempts to consume the given keyboard event.  If these
@@ -354,3 +387,83 @@ class KeyBindings:
                     debug.printException(debug.LEVEL_SEVERE)
 
         return consumed
+
+    def load(self, keymap, handlers):
+        """ Takes the keymappings and tries to find a matching named
+           function in handlers.
+           keymap is a list of lists, each list contains 5 elements
+           If addUnbound is set to true, then at the end of loading all the
+           keybindings, any remaining functions will be unbound.
+        """
+
+
+        for i in keymap:
+            keysymstring = i[0]
+            modifierMask = i[1]
+            modifiers = i[2]
+            handler = i[3]
+            try:
+                clickCount = i[4]
+            except:
+                clickCount = 1
+
+            if handlers.has_key(handler):
+                # add the keybinding
+                self.add(KeyBinding( \
+                  keysymstring, modifierMask, modifiers, \
+                    handlers[handler], clickCount))
+            else:
+                debug.println(debug.LEVEL_WARNING, \
+                  "WARNING: could not find %s handler to associate " \
+                  "with keybinding." % handler)
+
+    def validate(self):
+        """Tries to find keybindings where the keysym is not set for the
+        keyboard layout or where multiple keybindings map to the same
+        physical key."""
+
+        errorString = ""
+
+        # Find keybindings where the keysym is not found in the
+        # system's keymap.
+        #
+        for keyBinding in self.keyBindings:
+            keysymstring = keyBinding.keysymstring
+            if keysymstring:
+                if not getKeycode(keysymstring):
+                    errorString += "No physical key defines %s\n" \
+                                   % keysymstring
+                    if keyBinding.handler:
+                        errorString += "needed for %s\n" \
+                                       % keyBinding.handler.description
+
+        # Now, find duplicate bindings that are unintended duplicates.
+        #
+        for i in range(0, len(self.keyBindings)):
+            iKeyBinding = self.keyBindings[i]
+            if not iKeyBinding.keycode:
+                iKeyBinding.keycode = getKeycode(iKeyBinding.keysymstring)
+            if iKeyBinding.keycode:
+                for j in range(i + 1, len(self.keyBindings)):
+                    jKeyBinding = self.keyBindings[j]
+                    if not jKeyBinding.keycode:
+                        jKeyBinding.keycode = \
+                            getKeycode(jKeyBinding.keysymstring)
+                    if (iKeyBinding.keycode == jKeyBinding.keycode) \
+                       and (iKeyBinding.click_count \
+                            == jKeyBinding.click_count) \
+                       and (iKeyBinding.handler != jKeyBinding.handler) \
+                       and ((iKeyBinding.modifiers
+                             & iKeyBinding.modifier_mask) \
+                            == (jKeyBinding.modifiers
+                                & jKeyBinding.modifier_mask)):
+                        errorString += "%s maps to the same key as %s\n" \
+                                       % (iKeyBinding.keysymstring,
+                                          jKeyBinding.keysymstring)
+                        if iKeyBinding.handler:
+                            errorString += "used by %s\n" \
+                                           % iKeyBinding.handler.description
+                        if jKeyBinding.handler:
+                            errorString += "used by %s\n" \
+                                           % jKeyBinding.handler.description
+        return errorString
