@@ -103,6 +103,9 @@ if settings.debugMemoryUsage:
                  | gc.DEBUG_OBJECTS
                  | gc.DEBUG_SAVEALL)
 
+
+EXIT_CODE_HANG = 50
+
 # The user-settings module (see loadUserSettings).
 #
 _userSettings = None
@@ -1674,23 +1677,22 @@ def start(registry):
     registry.start(gil=settings.useGILIdleHandler)
 
 def die(exitCode=1):
+    pid = os.getpid()
+    if exitCode == EXIT_CODE_HANG:
+        # Someting is hung and we wish to abort.
+        os.kill(pid, signal.SIGKILL)
+        return
+
     shutdown()
     sys.exit(exitCode)
-
     if exitCode > 1:
-        pid = os.getpid()
-        if exitCode == 50:
-            # Something is hung and we wish to abort.
-            sig = 9
-        else:
-            sig = 15
-        os.kill(pid, sig)
+        os.kill(pid, signal.SIGTERM)
 
 def timeout(signum=None, frame=None):
     debug.println(debug.LEVEL_SEVERE,
                   "TIMEOUT: something has hung.  Aborting.")
     debug.printStack(debug.LEVEL_ALL)
-    die(50)
+    die(EXIT_CODE_HANG)
 
 def shutdown(script=None, inputEvent=None):
     """Exits Orca.  Unregisters any event listeners and cleans up.  Also
@@ -1786,7 +1788,7 @@ def shutdownOnSignal(signum, frame):
         signal.alarm(0)
 
     if not cleanExit:
-        die(signum)
+        die(EXIT_CODE_HANG)
 
 def abortOnSignal(signum, frame):
     debug.println(debug.LEVEL_ALL,
@@ -2162,20 +2164,26 @@ def main():
     from xdg.BaseDirectory import xdg_data_home
     userPrefsDir = os.path.join(xdg_data_home, "orca")
     oldUserPrefsDir = os.path.join(os.environ["HOME"], ".orca")
-    if os.path.exists(oldUserPrefsDir):
-        try:
-            shutil.copytree(oldUserPrefsDir, userPrefsDir)
-        except:
-            for name in os.listdir(oldUserPrefsDir):
-                srcPath = os.path.join(oldUserPrefsDir, name)
-                dstPath = os.path.join(userPrefsDir, name)
-                if os.path.isfile(srcPath):
-                    shutil.copy(srcPath, dstPath)
-                elif os.path.isdir(srcPath):
-                    if not os.path.isdir(dstPath):
-                        os.mkdir(dstPath)
-    settings.userPrefsDir = userPrefsDir
 
+    if not os.path.exists(userPrefsDir):
+        os.mkdir(userPrefsDir)
+
+    for baseDirName, dirNames, fileNames in os.walk(oldUserPrefsDir):
+
+        for dirName in dirNames:
+            relPath = os.path.relpath(baseDirName, oldUserPrefsDir)
+            dstDir = os.path.join(os.path.join(userPrefsDir, relPath), dirName)
+            if not os.path.exists(dstDir):
+                os.mkdir(dstDir)
+
+        for fileName in fileNames:
+            srcFile = os.path.join(baseDirName, fileName)
+            relPath = os.path.relpath(baseDirName, oldUserPrefsDir)
+            dstFile = os.path.join(os.path.join(userPrefsDir, relPath), fileName)
+            if not os.path.exists(dstFile):
+                shutil.copy(srcFile, dstFile)
+
+    settings.userPrefsDir = userPrefsDir
 
     # Do not run Orca if accessibility has not been enabled.
     # We do allow, however, one to force Orca to run via the
@@ -2233,7 +2241,10 @@ def main():
         else:
             _showPreferencesConsole()
 
-    start(pyatspi.Registry) # waits until we stop the registry
+    try:
+        start(pyatspi.Registry) # waits until we stop the registry
+    except:
+        die(EXIT_CODE_HANG)
     return 0
 
 if __name__ == "__main__":
