@@ -28,7 +28,7 @@ __license__   = "LGPL"
 
 # $ORCA_MAJOR_VERSION.$ORCA_MINOR_VERSION.$ORCA_MICRO_VERSION
 #
-version     = "2.31.92"
+version     = "2.91.0"
 
 # "--prefix" parameter used when configuring the build.
 #
@@ -44,4 +44,4 @@ datadirname = "share"
 
 # The directory where we could find liblouis translation tables.
 #
-tablesdir = "/usr/share/liblouis/tables"
+tablesdir = ""
