@@ -89,6 +89,7 @@ from orca_i18n import C_          # to provide qualified translatable strings
 userCustomizableSettings = [
     "orcaModifierKeys",
     "enableSpeech",
+    "onlySpeakDisplayedText",
     "speechServerFactory",
     "speechServerInfo",
     "voices",
@@ -188,9 +189,9 @@ userCustomizableSettings = [
     "brailleRequiredStateString",
     "speechRequiredStateString",
     "chatMessageVerbosity",
-    "chatPrefixMessageWithName",
+    "chatSpeakRoomName",
     "chatAnnounceBuddyTyping",
-    "chatSeparateRoomHistories",
+    "chatRoomHistories",
     "enableFlashMessages",
     "brailleFlashTime",
     "flashIsPersistent",
@@ -225,6 +226,10 @@ quitModule = "orca_quit"
 #
 findModule = "orca_gui_find"
 
+# The name of the module that holds the splash screen.
+#
+splashModule = "orca_gui_splash"
+
 # A list of keys that can serve as the Orca modifier key.  The list is
 # so we can provide better cross platform support (e.g., Sun keyboard
 # vs. PC-104 keyboard layouts).  When any of these keys is pressed,
@@ -360,6 +365,11 @@ enableSpeechCallbacks   = True
 #
 silenceSpeech           = False
 
+# If True, only text which is displayed on screen will be spoken
+# (roles, states, etc. will not be).
+#
+onlySpeakDisplayedText = False
+
 # Settings that apply to the particular speech engine to
 # use as well details on the default voices to use.
 #
@@ -373,11 +383,13 @@ speechServerInfo        = None # None means let the factory decide.
 DEFAULT_VOICE           = "default"
 UPPERCASE_VOICE         = "uppercase"
 HYPERLINK_VOICE         = "hyperlink"
+SYSTEM_VOICE            = "system"
 
 voices = {
     DEFAULT_VOICE   : ACSS({}),
     UPPERCASE_VOICE : ACSS({ACSS.AVERAGE_PITCH : 5.6}),
-    HYPERLINK_VOICE : ACSS({})
+    HYPERLINK_VOICE : ACSS({}),
+    SYSTEM_VOICE    : ACSS({}),
 }
 
 # If True, enable speaking of speech indentation and justification.
@@ -848,10 +860,12 @@ learnModeEnabled        = False
 #
 listShortcutsModeEnabled = False
 
-# The location of the user's preferences. By default this is ~/.orca.
+# The location of the user's preferences. By default this is
+# $XDG_DATA_HOME/orca.
 # It can be overridden by the Orca -d command line option.
 #
-userPrefsDir = os.path.join(os.environ["HOME"], ".orca")
+from xdg.BaseDirectory import xdg_data_home
+userPrefsDir = os.path.join(xdg_data_home, "orca")
 
 # Assists with dealing with CORBA COMM_FAILURES.  A failure doesn't
 # always mean an object disappeared - there just might be a network
@@ -885,7 +899,7 @@ useBlockPreventor       = False
 # If True, we use the bonobo main loop provided by bonobo to handle
 # all events in atspi.py.  If False, we create our own loop.
 #
-useBonoboMain           = True
+useBonoboMain           = False
 
 # If True, we handle events asynchronously - our normal mode of
 # queueing events and processing them later on the gidle thread.
@@ -1157,6 +1171,10 @@ setScriptMapping(re.compile('Shredder'), "Thunderbird")
 #
 setScriptMapping(re.compile(_('Mail/News')), "Thunderbird")
 
+# This is for a 3.1 version of Thunderbird, which chooses to now call itself
+# by a different name, Lanikai.
+setScriptMapping(re.compile('Lanikai'), "Thunderbird")
+
 # Translators: see the regular expression note above.  This is for
 # gnome_segv2, which calls itself bug-buddy in at-poke.
 #
@@ -1508,6 +1526,7 @@ TIME_FORMAT_24_HM = "%H:%M"
 # user may wish to be presented with.
 #
 TIME_FORMAT_24_HM_WITH_WORDS = _("%H hours and %M minutes.")
+TIME_FORMAT_CUSTOM = "%X"
 presentTimeFormat = TIME_FORMAT_LOCALE
 
 # Format directives to use in presentDate function.
@@ -1517,6 +1536,25 @@ presentTimeFormat = TIME_FORMAT_LOCALE
 # http://docs.python.org/library/time.html#time.strftime
 #
 DATE_FORMAT_LOCALE = "%x"
-DATE_FORMAT_WITH_LONG_NAMES = "%A, %d, %B, %Y."
-DATE_FORMAT_WITH_SHORT_NAMES = "%a, %d, %b, %Y."
+DATE_FORMAT_NUMBERS_DM = "%d/%m"
+DATE_FORMAT_NUMBERS_MD = "%m/%d"
+DATE_FORMAT_NUMBERS_DMY = "%d/%m/%Y"
+DATE_FORMAT_NUMBERS_MDY = "%m/%d/%Y"
+DATE_FORMAT_NUMBERS_YMD = "%Y/%m/%d"
+DATE_FORMAT_FULL_DM = "%A, %-d %B"
+DATE_FORMAT_FULL_MD = "%A, %B %-d"
+DATE_FORMAT_FULL_DMY = "%A, %-d %B, %Y"
+DATE_FORMAT_FULL_MDY = "%A, %B %-d, %Y"
+DATE_FORMAT_FULL_YMD = "%Y. %B %-d, %A."
+DATE_FORMAT_ABBREVIATED_DM = "%a, %-d %b"
+DATE_FORMAT_ABBREVIATED_MD = "%a, %b %-d"
+DATE_FORMAT_ABBREVIATED_DMY = "%a, %-d %b, %Y"
+DATE_FORMAT_ABBREVIATED_MDY = "%a, %b %-d, %Y"
+DATE_FORMAT_ABBREVIATED_YMD = "%Y. %b %-d, %a."
+
+# To keep Orca from spitting up upon launch.
+#
+DATE_FORMAT_WITH_LONG_NAMES = DATE_FORMAT_FULL_DMY
+DATE_FORMAT_WITH_SHORT_NAMES = DATE_FORMAT_ABBREVIATED_DMY
+
 presentDateFormat = DATE_FORMAT_LOCALE
